
/* This program demonstrates the functioning of a hotel light switch. 
 * When a button is pressed the light switches to full brightness, stays on full brightness for a time,
 * then slowly fades off. When the button is pressed during fading the light switches to full brightness again.
 *
 * Multitasking framework used is from paper by Loe Feijs entitled Multi-tasking and Arduino: Why and How?
 * Published in Chen, L.-L., T. Djajadiningrat, L. Feijs, S. Fraser, J. Hu, S. Kyffin and D. Steffen, Eds. (2013). 
 * Design and semantics of form and movement. 8th International Conference on 
 * Design and Semantics of Form and Movement (DeSForM 2013). ISBN 978-90-386-3462-3,  Wuxi, Philips.
 *
 * Software coding Jan Rouvroye for workshop multitasking Arduino.
 * Last edit: October 16, 2014
 */

#define LED_PIN 11                // pin where led is connected
#define BUTTON_PIN 12             // pin where button is connected

// states for led
#define STATE_LED_OFF 0           
#define STATE_LED_ON 1
#define STATE_LED_FADEOFF 2

// states for button
#define STATE_BUTTON_IDLE 0
#define STATE_BUTTON_DEBOUNCING 1

// other settings
#define DEBOUNCE_TIMER_SETTING  15   // timer setting for button debouncing
#define ONTIMER_SETTING 500          // timer setting for led on
#define FADETIMER_SETTING 2         // timer setting for fade off to dimmed (number of timesteps before next fade step)
#define FADESTEP 1                   // pwm setting for dimmed state

int ledState;                     // variable indicating the state of the led
boolean buttonEvent = false;      // indicator buttonpress has been registered after debouncing (true or false)

void setup() {
  Serial.begin(9600);      // for debugging

  pinMode(LED_PIN, OUTPUT);
  ledState = STATE_LED_OFF;
  analogWrite(LED_PIN, 0);

  pinMode(BUTTON_PIN, INPUT_PULLUP);
}

void loop() {
  buttonstep();
  ledStep();

  //printDebuginfo();
  delay(10);                      // loop once every 10 ms (if no other processes delay execution)
}

void buttonstep() {          // reads button, performs debouncing and sets buttonEvent indicator accordingly
  int buttonValue;
  static int buttonState = STATE_BUTTON_IDLE;                  // buttonstate indicator
  static int buttonPressedcounter;
  static int debounceTimer;

  buttonValue = digitalRead(BUTTON_PIN);

  buttonEvent = false;
  switch(buttonState)
  {
  case STATE_BUTTON_IDLE:
    if (buttonValue == LOW)      //remember LOW means button is being pressed
    {
      buttonPressedcounter = 1;

      buttonState = STATE_BUTTON_DEBOUNCING;
      debounceTimer = DEBOUNCE_TIMER_SETTING;
    }
    break;

  case STATE_BUTTON_DEBOUNCING:
    if (buttonValue == LOW)
      buttonPressedcounter++;

    debounceTimer--;
    if (debounceTimer < 0)        // timer expired
    {
      buttonState = STATE_BUTTON_IDLE;

      if (buttonPressedcounter > DEBOUNCE_TIMER_SETTING/2)    // button is assumed to be pressed when at least half of the time of the debouncing period it was pressed
        buttonEvent = true;
    }

    break;
  }
}

void ledStep() {        // handles the led depending on the led state 
  static int onTimer;          // timer for led on full brightness
  static int fadeTimer;        // timer for fading 
  static int currentPwm = 0;    // pwm setting for led (0-255)

  switch(ledState) {
  case STATE_LED_OFF:
    currentPwm = 0;

    if (buttonEvent == true)
    {
      ledState = STATE_LED_ON;
      onTimer = ONTIMER_SETTING;
    }
    break;

  case STATE_LED_ON:
    currentPwm = 255;

    if (onTimer > 0)
      onTimer = onTimer - 1;
    else
    {
      ledState = STATE_LED_FADEOFF;
      fadeTimer = FADETIMER_SETTING;
    }
    break;

  case STATE_LED_FADEOFF:
    if (fadeTimer > 0)
      fadeTimer = fadeTimer - 1;
    else
    {
      fadeTimer = FADETIMER_SETTING;
      currentPwm = currentPwm - FADESTEP;
      if (currentPwm < 0)
      {
        currentPwm = 0;
        ledState = STATE_LED_OFF;
      }
    }

    if (buttonEvent == true)
    {
      ledState = STATE_LED_ON;
      onTimer = ONTIMER_SETTING;
    }
    break;
  }
  analogWrite(LED_PIN, currentPwm);
}

void printDebuginfo()
{
  Serial.print("buttonEvent: "); 
  Serial.print(buttonEvent); 
  Serial.print("\t");
  Serial.print("\t");
  Serial.print("ledState: "); 
  Serial.print(ledState); 
  Serial.print("\t");
  Serial.println();
}

























